@@ -40,7 +40,8 @@ our @EXPORT = qw(
 
 );
 
-$VERSION = '0.51';
+#our $VERSION = '0.50';
+$VERSION = '0.50';
 
 our $DEBUGTYPE = 'apache';
 
@@ -74,7 +75,6 @@ sub display {
 	$dbh = shift;
 	$r = shift;
 	my $authsuperuserdir = shift;
-	my $cookieheader = shift;
 	my @commandlist = @_;
 
 	# These requests are special, and might want to use a HTTP redirect, so we can't 
@@ -83,14 +83,14 @@ sub display {
 	my ($result,$errorcode) = ("",0);
 	my $uri = $r->uri;
 	for ($uri) {
-		/\/jadd.wsql$/				&& do { ($result,$errorcode) = &jAddPrepare($dbh,$cookieheader); last; };
-		/\/jupdate.wsql$/			&& do { ($result,$errorcode) = &jUpdatePrepare($dbh,$cookieheader); last; };
-		/\/jdelete.wsql$/			&& do { ($result,$errorcode) = &jDeletePrepare($dbh,$cookieheader); last; };
-		/\/jform.wsql$/				&& do { ($result,$errorcode) = &jForm($dbh,$cookieheader); last; };
-		/\/jdeleteform.wsql$/	&& do { ($result,$errorcode) = &jDetails($dbh,$cookieheader); last; };
-		/\/jdetails.wsql$/		&& do { ($result,$errorcode) = &jDetails($dbh,$cookieheader); last; };
-		/\/jlist.wsql$/				&& do { ($result,$errorcode) = &jList($dbh,0,$cookieheader); last; };
-		/\/jloginform.wsql$/	&& do { ($result,$errorcode) = &jLoginForm($dbh,$cookieheader); last; };
+		/\/jadd.wsql$/				&& do { ($result,$errorcode) = &jAddPrepare($dbh); last; };
+		/\/jupdate.wsql$/			&& do { ($result,$errorcode) = &jUpdatePrepare($dbh); last; };
+		/\/jdelete.wsql$/			&& do { ($result,$errorcode) = &jDeletePrepare($dbh); last; };
+		/\/jform.wsql$/				&& do { ($result,$errorcode) = &jForm($dbh); last; };
+		/\/jdeleteform.wsql$/	&& do { ($result,$errorcode) = &jDetails($dbh); last; };
+		/\/jdetails.wsql$/		&& do { ($result,$errorcode) = &jDetails($dbh); last; };
+		/\/jlist.wsql$/				&& do { ($result,$errorcode) = &jList($dbh); last; };
+		/\/jloginform.wsql$/	&& do { ($result,$errorcode) = &jLoginForm($dbh); last; };
 		/\/jlogin.wsql$/			&& do { ($result,$errorcode) = &jLogin($dbh,$authsuperuserdir); last; };
 		/\/jlogout.wsql$/			&& do { ($result,$errorcode) = &jLogout($dbh); last; };
 	}
@@ -98,24 +98,18 @@ sub display {
 	if ($result eq "") {
 		my $dd = localtime();
 		# Print a proper HTTP header
-		print <<EOF;
+	  print <<"EOL";
 HTTP/1.1 200 OK
 Date: $dd
 Server: Apache
-EOF
-		print "$cookieheader\r\n" if (defined($cookieheader) && ($cookieheader ne ''));
-		print <<EOF;
 Connection: close
 Content-type: text/html
 
-EOF
+EOL
 		# Note that the next line will make requests ending in a slash look for a index.wsql file...
 		# So if you want to link to a index.html file, you'll have to put the file name in the request!
 		$uri .= "index.wsql" if ($uri =~ /\/$/);
-
-		my $doc_root = $r->document_root;
-
-		$result = &readWeSQLFile($doc_root . $uri);
+		$result = &readWeSQLFile($r->document_root . $uri);
 	}
 
 	# Now only parse files as WeSQL if they have a .wsql suffix!! This allows easy mixing of HTML and WeSQL files...
@@ -143,7 +137,7 @@ sub error {
 <h1>Error</h1>
 $message
 <hr>
-This page was dynamically generated by <a href=http://wesql.org>WeSQL $VERSION</a>
+This page was dynamically generated by <a href=http://wesql.org>WeSQL</a>
 </body>
 </html>
 EOF
@@ -277,12 +271,9 @@ sub readLayoutFile {
 	# Get the 'base-uri' from the request: for instance, for /admin/jlist.wsql that would be /admin/, and for /jlist.wsql that would just be /
 	my $uri = $r->uri;
 	my ($baseuri) = ($uri =~ /^(.+)\//);
-
-	my $doc_root = $r->document_root;
-
 	$baseuri .= '/';
-	if (!defined(open(LAYOUT,$doc_root . $baseuri . "$file"))) {
-		&log_error("$$: WeSQL.pm: readLayoutFile: file '" . $doc_root . $baseuri . "$file' not found!");
+	if (!defined(open(LAYOUT,$r->document_root . $baseuri . "$file"))) {
+		&log_error("$$: WeSQL.pm: readLayoutFile: file '" . $r->document_root . $baseuri . "$file' not found!");
 		&jErrorMessage("Configuration file not found! Please contact the webmaster.","Can't read $file!",0);
 		exit;
 	}
@@ -407,22 +398,16 @@ sub dosubst_inline {
 # doeval deals with the <!-- EVAL FLAG ... /EVAL FLAG --> tag
 ############################################################
 
-# We need evalinline to be able to trap the eval errors...
-# $@ gets lost in the s//eval()/ statement in doeval.
-sub evalinline {
-	my $eval = eval($_[0]);
-	&log_error("$$: DOEVAL EVAL ERROR: " . $@) if $@;  #This will log errors from the eval() 
-	return $eval;
-}
-
 sub doeval {
-	my $body = shift;
-	my $param = shift;
-	# And then the single-line style evals (e.g. <!-- EVAL POST (some_perl_code) -->)
-	$body =~ s/<!-- EVAL $param (.+?) -->/&evalinline($1)/emg;
+  my $body = shift;
+  my $param = shift;
 	# First do the multi-line evals
-	$body =~ s/<!-- EVAL $param\s*\n(.*?)^\/EVAL $param -->/&evalinline($1)/esmg;
-	return $body;
+  $body =~ s/<!-- EVAL $param (.+?) -->/eval($1)/emg;
+  &log_error("$$: doeval eval error: " . $@) if $@;  #This will log errors from the eval() 
+	# And then the single-line style evals (e.g. <!-- EVAL POST (some_perl_code) -->)
+  $body =~ s/<!-- EVAL $param\s*\n(.*?)^\/EVAL $param -->/eval($1)/esmg;
+  &log_error("$$: doeval eval error: " . $@) if $@;  #This will log errors from the eval() 
+  return $body;
 }
 
 ############################################################
@@ -493,6 +478,7 @@ sub check_one_param {
 	$negation = '' if (!defined($negation));
 	$regexp = '' if (!defined($regexp));
 	$wrong_string = '' if (!defined($wrong_string));
+	print "param: $param, negation: $negation, regexp: $regexp, wrong_string: $wrong_string<p>";
 	chop($negation);
 	if ($negation =~ /^!/) {
 		if ($regexp eq '') { # !// means that the parameter should be defined
@@ -587,17 +573,11 @@ sub getparams {
 }
 
 ############################################################
-# redirect
-# redirect redirects a browser to a new location. Parameters:
-# destination (required), and any number of valid header lines (optional)
+# redirect redirects a browser to a new location
 ############################################################
 sub redirect {
-	my $dest = shift;
 	print "HTTP/1.0 302 Redirect\r\n";
-	print "Location: $dest\r\n";
-	foreach (@_) {
-		print "$_\r\n";
-	}
+	print "Location: $_[0]\r\n";
 	print "Content-type:text/html\r\n\r\n";
 	print << "EOF";
 <!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
@@ -605,7 +585,7 @@ sub redirect {
 <TITLE>302 Found</TITLE>
 </HEAD><BODY>
 <H1>Found</H1>
-The document has moved <A HREF="$dest">here</A>.<P>
+The document has moved <A HREF="$_[0]">here</A>.<P>
 <HR>
 <ADDRESS>Apache Server</ADDRESS>
 </BODY></HTML>
@@ -718,7 +698,7 @@ on my system: /usr/local/lib/perl5/site_perl/5.6.1/Apache/WeSQL/AppHandler.pm
 =item *
 
 Now edit AppHandler2.pm and replace all occurences of the string 'AppHandler'
-with 'AppHandler2'. There should be 4 occurences in v0.5x. If you use vi/vim, you can
+with 'AppHandler2'. There should be 4 occurences in v0.50. If you use vi/vim, you can
 use this command: :%s/AppHandler/AppHandler2/g
 
 =item *
@@ -998,10 +978,10 @@ Say we have a WeSQL file that looks like this:
 
     <html>
     <body>
-    <!-- PARAMCHECK
-    <paramcheckhead>
+    <!-- PARAMCHECK Below are the problems that have been encountered:<p>
+    <PARAMCHECKHEAD>
     <center>Below are the problems that have been encountered:<p>
-    </paramcheckhead>
+    </PARAMCHECKHEAD>
     PR_ONE      /STOP/          <font color=#FF0000>Parameter 'one' must contain the word 'STOP'</font><br>
     PR_TWO      /^\d+$/         <font color=#FF0000>Parameter 'two' must be a number and may not be empty</font><br>
     PR_THREE    //              <font color=#FF0000>Parameter 'three' must be defined!</font><br>
@@ -1009,11 +989,11 @@ Say we have a WeSQL file that looks like this:
     PR_THREE    !/%/            <font color=#FF0000>Parameter 'three' can not contain a %-sign!</font><br>
     PR_FOUR     !/[\/ ]/        <font color=#FF0000>Parameter 'four' can not contain a forward slash or a space!</font><br>
     PR_FIVE     !//             <font color=#FF0000>Parameter 'five' may not be defined!</font><br>
-    <paramcheckfoot>
+    <PARAMCHECKFOOT>
     </center>
     <!-- INCLUDE footer.wsql -->
     <!-- CUTFILE -->
-    </paramcheckfoot>
+    </PARAMCHECKFOOT>
     /PARAMCHECK -->
     <p>
     If you can read this, all parameters conform with the conditions.
@@ -1227,10 +1207,6 @@ Yes. But, of course, the latter is lots faster, especially on large tables... So
 
 And this would have the desired results, depending on whether name is defined as a parameter or not.
 
-This module is part of the WeSQL package, version 0.51
-
-(c) 2000-2002 by Ward Vandewege
-
 =head1 EXPORT
 
 None by default. Possible: %params %cookies &redirect &error &readLayoutFile
@@ -1246,7 +1222,7 @@ I am very grateful to the both of them.
 
 Ward Vandewege, E<lt>ward@pong.beE<gt>
 
-Copyright (c) 2000-2002 Ward Vandewege. This program is free software; you can redistribute it and/or modify it under the terms of the GPL.
+Copyright (c) 2000-2001 Ward Vandewege. This program is free software; you can redistribute it and/or modify it under the terms of the GPL.
 
 =head1 SEE ALSO
 
